#!/bin/bash

# IB Check Cluster MPI tests
# Sourced by ib-check-cluster

function ib-check-cluster-mpi-seq-pairs-bi-directional-bw()
{
    echo "Bi directional test of seq pairs, loop: ${bw_num_loops}, msg-size: ${bw_msg_size}"
    for (( i = 0; i + 1 < ${#H[@]}; i++)); do
        #  echo $i ${H[$i]} ${H[$(( (i + 1) % ${#H[@]} ))]}
        h1=${H[$i]}
        h2=${H[$(( i + 1 ))]}

        ${mpi_cmd} -noinput -np 2 $h1 $h2 /usr/voltaire/mpi/bin/mpi_bandwidth ${bw_num_loops} ${bw_msg_size} > ${tempdir}/ib-check-$$-c-pm-seq-$h1-$h2.log 2>&1 &
        ${mpi_cmd} -noinput -np 2 $h2 $h1 /usr/voltaire/mpi/bin/mpi_bandwidth ${bw_num_loops} ${bw_msg_size} > ${tempdir}/ib-check-$$-c-pm-seq-$h2-$h1.log 2>&1 & 
        wait
    done
}
function ib-check-cluster-mpi-seq-pairs-uni-directional-bw()
{
    echo "Uni directional test of seq pairs, loop: ${bw_num_loops}, msg-size: ${be_msg_size}"
    for (( i=0; i + 1 < ${#H[@]}; i += 2)); do
        h1=${H[$i]}
        h2=${H[$(( (i + 1) % ${#H[@]} ))]}
	${mpi_cmd} -noinput -np 2 $h1 $h2 /usr/voltaire/mpi/bin/mpi_bandwidth ${bw_num_loops} ${bw_msg_size} > "${tempdir}"/ib-check-$$-${h1}-${h2}-uni-bw.log 2>&1 &
    done
}
function ib-check-cluster-mpi-sinle-seq-pair-bw()
{
    for (( i=0; i + 1 < ${#H[@]}; i += 2)); do
        h1=${H[$i]}
        h2=${H[$(( (i + 1) % ${#H[@]} ))]}
	bw=$(${mpi_cmd} -np 2 $h1 $h2 /usr/voltaire/mpi/bin/mpi_bandwidth ${bw_num_loops} ${bw_msg_size} 2>/dev/null | sed -e 's/^.*     //' -e 's/\..*//')
	if [[ ${bw} -lt 700 ]]; then
	    msg-red "[$h1] -> [$h2] Low bandwidth ($bw)"
	fi
    done
}
function ib-check-cluster-mpi-scale()
{
    local cpu_number=$( ${rsh_cmd} ${H[0]]} 'cat /proc/cpuinfo | grep processor | wc -l | sed "s/ //g"' )
    echo -n > "${tempdir}"/ib-check-hosts1.txt
    echo -n > "${tempdir}"/ib-check-hosts2.txt
    echo -n > "${tempdir}"/ib-check-hosts4.txt
    echo -n > "${tempdir}"/ib-check-hosts8.txt

    for (( i = 0; i < ${#H[@]}; i++ )); do
	echo ${H[${i}]} >> "${tempdir}"/ib-check-hosts1.txt
	echo ${H[${i}]} >> "${tempdir}"/ib-check-hosts2.txt
	echo ${H[${i}]} >> "${tempdir}"/ib-check-hosts2.txt
    	echo ${H[${i}]} >> "${tempdir}"/ib-check-hosts4.txt
	echo ${H[${i}]} >> "${tempdir}"/ib-check-hosts4.txt
    	echo ${H[${i}]} >> "${tempdir}"/ib-check-hosts4.txt
	echo ${H[${i}]} >> "${tempdir}"/ib-check-hosts4.txt
    done
    ${mpi_cmd} -noinput -np ${#H[@]} -hostfile "${tempdir}"/ib-check-hosts1.txt /usr/voltaire/mpi/bin/cscale > ib-check-cls-mpi-scale1.log 2>&1
    ${mpi_cmd} -noinput -np $(( 2 * ${#H[@]})) -hostfile "${tempdir}"/ib-check-hosts1.txt /usr/voltaire/mpi/bin/cscale > ib-check-cls-mpi-scale2.log 2>&1
}
function ib-check-cluster-mpi-stress()
{
    ${mpi_cmd} -noinput -np ${#H[@]} -hostfile "${tempdir}"/ib-check-hosts1.txt /usr/voltaire/mpi/bin/cstress > ib-check-cls-mpi-stress1.log 2>&1
}
function ib-check-cluster-mpi-tests()
{
    echo -n "MPI Bandwidth "

    ib-check-cluster-mpi-sinle-seq-pair-bw

    echo -n "MPI Scaling "

    ib-check-cluster-mpi-scale

    echo -n "MPI Stressing "

    ib-check-cluster-mpi-stress
}
