#!/bin/bash
# Created 19/11/2003
# Updated may 2005
# Updated 10/26/2005

function ib-local-info()
{
    # Run vstat and analyze its output:
    
    if [[ -e /usr/mellanox/bin/vstat ]]; then
	ibstatus="$(/usr/mellanox/bin/vstat)"
        ib-vstat-parse-info $(hostname) "$ibstatus"
    elif [[ -e /usr/bin/vstat ]]; then
	ibstatus="$(/usr/bin/vstat)"
        ib-vstat-parse-info $(hostname) "$ibstatus"
    else
	msg-red Could not find the vstat command, please verify your IB stack installation
    fi
}

function ib-check-local-perf-main-loopback-lat()
{
    perf_main --send -mlat -rwrite -s1 -n5000 > "${tempdir}"/ib-check-$$-local-perf-main-loopback-lat.log 2>&1 &
    sleep 1
    perf_main -a 127.0.0.1 > /dev/null 2>&1 &
    wait
    msg-white "Loopback $(grep '^Lat' "${tempdir}"/ib-check-$$-local-perf-main-loopback-lat.log)"
}

function ib-check-local-perf-main-loopback-bw()
{
    perf_main --send -mbw -rwrite -s512000 -n500 > "${tempdir}"/ib-check-$$-local-perf-main-loopback-bw.log 2>&1 &
    sleep 1
    perf_main -a 127.0.0.1 > /dev/null 2>&1 &
    wait
    msg-white "Loopback bandwidth $(grep '^BW:' "${tempdir}"/ib-check-$$-local-perf-main-loopback-bw.log)"
}

function ib-check-local-host()
{
    # Run the following tests on the local host:
    # 1. Check for ibhost-release file content (rpm -V ?)
    # 1.1 Check 'rpm -V ibhost-biz/hpc 2>&1 | grep -v repository | grep -v mst.conf'
    # 2. lspci
    # 3. ibhost.init reset
    # 4. vstat
    # 5. IPoIB address & ping
    # 6. perf_main
    #
    msg-white "Voltaire InfiniBand Stack - Local Host Verification"
    msg-white hostname $(hostname)

    local rc=0 ; # Return Code by default 0 - OK.

    # Check ibhost-*.rpm installation
    if [[ $rpm_check -eq 1 ]]; then
        rpm_package="$(rpm -qa | grep ibhost)"
        if [[ $(echo "$rpm_package" | wc -l) -eq 1 ]]; then
            msg-green "Found $rpm_package installed"
            rpm_v="$(rpm -V $rpm_package 2>&1 | grep -v repository | grep -v mst.conf)"
            if [[ $(echo "$rpm_v" | wc -c) -gt 6 ]]; then
                msg-red "Warning: The following files of the $rpm_package were modifyed: $rpm_v"
            fi
        else 
            msg-red "Error: Number of ibhost packages is *not* one: $rpm_package."
            return 1
        fi
    fi
    # Verify we see the HCA on the PCI bus(es)
    if lspci | grep -q Mellanox ; then
        msg-green Mellanox HCA found in system
	# If PCI-X verify two devices, and PCI-E one.
        # Verify 3 PCI BARs with their sizes
    else
        msg-red 'Mellanox HCA was not found in the system, exit.'
        exit 1
    fi

    # Test retesting the IB stack
    # ibhost.init is writing into /dev/tty - so we need to capture the tty output,
    #  we use the script command for this.
    if [[ ${use_script} -ne 0 && -e /usr/bin/script ]]; then
        script -c "ibhost.init restart" /dev/stdout
    else
        ibhost.init restart
    fi

    sleep 2
    if ! dmesg | grep -i thh | grep -iq failed ; then
        msg-green 'dmesg shows no errors on stack restart (load)'
    else
        msg-red Error: dmesg shows errors on stack restart, please use dmesg to get details
        msg-red Error: dmesg shows there are errors restarting the IB host stack.  Please review.  When the issue is resolved you can run ib-check.sh again.
    fi

    # Verify that the IPoIB module was loaded
    if lsmod | awk '/^ipoib.ud/{print $1}' | grep -q ipoib.ud ; then
        msg-green "IPoIB kernel module found"
    else
        msg-red "Error: IPoIB kernel module was *not* found"
        (( rc++ ))
    fi
    # Verify SDP
    # grep $(sdp-... /usr/voltaire/config/repository.rps | wc enaled | wc -l ) -eq 1
    # if lsmod | awk '/^sdp/{print $1}' | grep -q sdp ; then
    #     msg-green "SDP kernel module found"
    # else
    #     msg-red "Error: SDP kernel module was *not* found"
    # fi

    N=1
    while (( N < 10 )) ; do
        sleep 1;
        ipoib_ipaddr=$(/sbin/ifconfig ipoib0 | awk '/inet addr:/{print $2}' | sed s/addr://)
        if echo "${ipoib_ipaddr}" | egrep -q '^[0-9]{1,3}(\.[0-9]{1,3}){3}$' ; then
            msg-green "IPoIB interface 0 (ipoib0) IP address ${ipoib_ipaddr}."
            break
        else
            (( N++ ))
        fi
    done
    if (( N >= 10 )); then
        msg-red "IPoIB interface 0 (ipoib0) IP address is not defined '$ipoib_ipaddr'"
    fi
    ib-local-info

    ib-check-local-perf-main-loopback-bw
    ib-check-local-perf-main-loopback-lat

    return $rc
}

ib-check-local-host
