#!/bin/bash
# Written by Voltaire Support
# May 21 2005
# Updated 10/28/2005
# For problems with these scripts please contact Voltaire Support
# support@voltaire.com

# set -e            ;# Exit on any command error.

mode=single       ;# single | cluster | create_list
display=on        ;# default on, otherwise output to ib-check-output.log
basedir=.         ;# Later on change to search the executable.
tempdir=/tmp      ;# Temporary files
# readme=/usr/voltaire/docs/ib-check.README.txt
readme=./ib-check.README.txt
rsh_cmd=ssh       ;# ssh / rsh command
mpi_cmd=mpirun_ssh ;# mpirun_ssh / mpirun_rsh
vmpi_tests=0      ;# don't run Voltaire MPI tests.
head_ib=1         ;# Test IB on head node in case of cluster testing
nodes_file=./nodes.conf
rpm_check=1       ;# By default Verify the ibhost-*.rpm installation.
min_bandwidth=650 ;# minimum acceptable bandwidth [MB/s] of perf_main or mpi_bandwidth
bw_msg_size=500000 ;# MPI Bandwidth test - message size
bw_num_loops=200   ;# MPI Bandwidth test - number of loops
use_script=1      ;# Use the script command to capture the output of the ibhost.init start/stop

# Search and load configuration file before command line processing as
#  command line arguments should override the default.

if [[ -f "${basedir}/ib-check.conf" ]]; then
    . "${basedir}/ib-check.conf"
fi
if [[ "${basedir}/ib-check.conf" != ./ib-check.conf && -f ./ib-check.conf ]]; then
    . ./ib-check.conf
fi
if [[ -f nodes.conf ]]; then
    nodes_file=./nodes.conf
elif [[ -f "${basedir}/nodes.conf" ]]; then
    nodes_file="${basedir}/nodes.conf"
fi

if ! script -c date /dev/null > /dev/null 2>&1 ; then
    use_script=0
fi    
# Process command line arguments [--cluster] [--display-on] ...
while [[ "X$1" != X ]]; do
    case "$1" in
    --cluster)
        mode=cluster
    ;;
    --create-list)
        mode=create_list
    ;;
    --display-on)
        display=on
    ;;
    --display-off)
        display=off
    ;;
    --nodes-file)
        shift
        nodes_file="$1"
    ;;
    --ssh)
        rsh_cmd=ssh
        mpi_cmd=mpirun_ssh
    ;;
    --rsh)
        rsh_cmd=rsh
        mpi_cmd=mpirun_rsh
    ;;
    --vmpi)
        vmpi_tests=1
    ;;
    --skip-head)
        head_ib=0
    ;;
    --skip-rpm)
        rpm_check=0
    ;;
    --help|-h|\?)
        echo "usage: $0 [--cluster] [--create-list] [options]"
        echo "  --cluster  - test also the cluster"
        echo "  --display-on  - show sub programs output"
        echo "  --display-off - all output goes into ib-check-output.log"
        echo "  options - see the following readme file ..."
        if [[ -f $readme ]]; then
            echo " ---- Here is the readme file: $readme ----"
            cat $readme
        fi
        exit 0
    ;;
    *)
        echo "unknown option/argument $1, ignored"
    esac
    shift
done

basedir=$(dirname $0)

if [[ ! -e $0 ]]; then
    echo "Please run the program ($0) from its directory"
    exit 1
fi


if [[ $display == off ]]; then
    exec > ib-check-output.log 2>&1
fi

. ${basedir}/ib-check-functions.sh

# rm -rf *.txt *.log

local-mode()
{
    #need to add switch for user to run with minism
    . ${basedir}/ib-check-local.sh
}

nodes-list-create()
{
    echo Please enter IB node names seperated by spaces
    read list
    rm -rf nodes.conf
    for x in $list 
    do
        echo $x >> nodes.conf
    done
    nodes_file=./nodes.conf
}

cluster-mode()
{
    # By default do local check before cluster check, unless --skip-local
    if [[ $head_ib -eq 1 ]]; then
        . ${basedir}/ib-check-local.sh
    fi
    if [[ ! -f "${nodes_file}" ]]; then
        msg-red "Could not find nodes.conf list file '${nodes_file}',"
        msg-red " please use --create-list to create one, exit"
        exit 1
    fi
    msg-white "Found nodes.conf file '${nodes_file}'"
    . ${basedir}/ib-check-cluster.sh
}

# main execution

case $mode in
    single)
        local-mode
    ;;
    cluster)
        cluster-mode
    ;;
    create_list)
        nodes-list-create
    ;;
    *)
        echo Unknown run mode $mode, exit
        exit 1
    ;;
esac 

exit 0
